import os
from twitchio.ext import commands, routines
from shinyOdds import probShiny, findProb
import re

class Bot(commands.Bot):

    def __init__(self, *args, **kwargs):
        # Initialise our Bot with our access token, prefix and a list of channels to join on boot...
        super().__init__(*args, **kwargs)

    async def event_ready(self):
        # We are logged in and ready to chat and use commands...
        print(f'Logged in as | {self.nick}')
        print(f'User id is | {self.user_id}')

    @commands.command()
    async def hello(self, ctx: commands.Context):
        # Send a hello back!
        await ctx.send(f'Hello {ctx.author.name.strip()}!')

    @commands.command()
    async def shinyCalc(self, ctx:commands.Context):
        pattern = re.compile("!shinyCalc [0-9]+/([+-]?(?=\\.\\d|\\d)(?:\\d+)?(?:\\.?\\d*))(?:[eE]([+-]?\\d+))? [0-9]+")
        msg = ctx.message.content
        if pattern.fullmatch(msg) is None:
            await ctx.send(f'{ctx.author.name.strip()}, Error invalid usage. The command should be something like '
                           f'\"!shinyCalc 1/4096 725\"')
        else:
            msg = msg.split(' ')
            nEncounters = int(msg[2])
            odds = msg[1]
            odds = odds.split('/')
            odds = float(odds[0]) / float(odds[1])
            await ctx.send(f'{ctx.author.name.strip()}, your probability to have found a shiny by {nEncounters} '
                           f'encounters is {100*probShiny(nEncounters, odds, 1):.2f}%.')

    @commands.command()
    async def findEncounters(self, ctx:commands.Context):
        pattern = re.compile(
            "!findEncounters [0-9]+/([+-]?(?=\\.\\d|\\d)(?:\\d+)?(?:\\.?\\d*))(?:[eE]([+-]?\\d+))? [0-9]*\\.[0-9]+[%]?")
        msg = ctx.message.content
        if pattern.fullmatch(msg) is None:
            await ctx.send(f'{ctx.author.name.strip()}, Error invalid usage. The command should be something like '
                           f'\"!shinyCalc 1/4096 725\"')
        else:
            msg = msg.split(' ')
            wantedOdds = float(msg[2].replace("%",""))/100
            odds = msg[1]
            odds = odds.split('/')
            odds = float(odds[0]) / float(odds[1])
            nenc, prob = findProb(wantedOdds, odds=odds, nShines=1)
            await ctx.send(f'{ctx.author.name.strip()}, You\'ll need {nenc} encounters to reach {100*wantedOdds:.2f}%')

    @commands.command()
    async def help(self, ctx:commands.Context):
        await ctx.send(f'{ctx.author.name.strip()}, please go to https://bit.ly/3DyFkYh for a list of commands.')

    @routines.routine(hours=1)
    async def repos(self):
        msg = "Want to use the programs I'm using? Overlay items/bots: https://github.com/drkspace/Pokemon-Stream-Tools. The reset bot: https://github.com/brianuuu/AutoController_swsh. "
        await self._ws.send_privmsg(self.connected_channels[0],msg)



import scipy.stats as stats
import functools


@functools.lru_cache(maxsize=128)
def probShiny(nEncounters, odds, nShines):
    return 1 - stats.binom.cdf(k=nShines - 1, n=nEncounters, p=odds)


@functools.lru_cache(maxsize=128)
def findProb(wantedP, odds, nShinies):
    minVal = 0
    maxVal = 2 ** 30  # About 1 billion, but this will have the same runtime
    n = int((minVal + maxVal) / 2)
    while not (probShiny(n - 1, odds, nShinies) < wantedP <= probShiny(n, odds, nShinies)):

        if probShiny(n, odds, nShinies) > wantedP:
            maxVal = min(maxVal, n)

        else:
            minVal = max(minVal, n)

        n = int((minVal + maxVal) / 2)

    return n, probShiny(n, odds, nShinies)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import time
from obswebsocket import obsws, requests
import argparse
import os
import getpass


def parse_args():
    parser = argparse.ArgumentParser(description='Stream Ender.')
    parser.add_argument('encounter_fname', metavar='Encounters.txt', type=str,
                        help='The file location for the number of encounters')
    parser.add_argument('hostname', metavar='OBS_IP', type=str,
                        help='The IP of the OBS instance')
    parser.add_argument('port', metavar='PORT', type=int,
                        help='The port for the OBS instance')
    parser.add_argument('--password', nargs='?', type=str, default=None,
                        help='The password for the OBS instance')
    parser.add_argument('--timeout', nargs='?', const=300, type=float, default=300,
                        help='The time (in seconds) to start the \"End Stream\" screen')
    parser.add_argument('--EndStreamTime', nargs='?', const=60, type=float, default=60,
                        help='The time (in seconds) to wait on the end screen')
    parser.add_argument('--EndStreamName', nargs=1, type=str,
                        help='The name for the end stream scene')
    return parser.parse_args()


def run(host, port, password, fileName, timeout, endStreamTime, endStreamName):
    ws = obsws(host, port, password)
    # ws = obsws(host, port)
    ws.connect()

    while time.time() - os.path.getmtime(fileName) < timeout:
        time.sleep(timeout - (time.time() - os.path.getmtime(fileName)))

    try:
        print("Switching Scenes")
        ws.call(requests.SetCurrentScene(endStreamName))
        print("Waiting to end stream")
        time.sleep(endStreamTime)
        print("Stopping Streaming")
        ws.call(requests.StopStreaming())
        print("Streaming Stopped")
    except Exception as e:
        print(e)
    finally:
        ws.disconnect()


def main():
    args = parse_args()
    host = args.hostname
    port = args.port
    if args.password is None:
        password = getpass.getpass()
    else:
        password = args.password

    run(host, port, password, args.encounter_fname, args.timeout, args.EndStreamTime, args.EndStreamName)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding: utf-8

import math
import matplotlib.pyplot as plt
import numpy as np
import os
import datetime
import time
import matplotlib.ticker as mtick
import argparse
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler
from typing import List
from .calcShiny import probShiny

plt.rcParams.update({'font.size': 22})


def parse_arguments():
    parser = argparse.ArgumentParser(description='Time to shiny Hunt.')
    parser.add_argument('encounter_fname', metavar='Encounters.txt', type=str,
                        help='The file location for the number of encounters')
    parser.add_argument('odds', metavar='Odds', type=str,
                        help='The odds to find a shiny (i.e. 1/4096)')

    parser.add_argument('--nShinies', nargs='?', const=1, type=int, default=1,
                        help='The number of shinies you\'re hunting for')

    return parser.parse_args()


class ShinyCounterHandler(PatternMatchingEventHandler):
    patterns: List = ["*"]

    fig = plt.figure(figsize=(8, 8))
    ax = fig.add_subplot(111)
    nEncounters = -1

    def __init__(self, args):
        self.nShines = args.nShinies
        self.odds = args.odds.split('/')
        self.odds = float(self.odds[0]) / float(self.odds[1])
        self.fname = args.encounter_fname
        self.patterns[0] = self.fname
        self.n = np.array(range(int(10 * self.nShines / (self.odds))))
        self.probs = np.array([probShiny(_n, self.odds, self.nShines) for _n in self.n])
        self.comb = np.column_stack((self.n, self.probs))
        self.probs *= 100

        self.oddsStr = self.odds.as_integer_ratio()
        if self.oddsStr[1] > 1000000:
            self.oddsStr = [1, self.odds ** -1]

        self.oddsStr = f"{self.oddsStr[0]}/{self.oddsStr[1]}"
        self.lastModified = 0
        super(ShinyCounterHandler, self).__init__()

    def getEncounters(self) -> int:
        if os.path.exists(self.fname):
            with open(self.fname, 'r') as f:
                nEncounters = int(f.readline().split(' ')[1])
        return nEncounters

    def on_modified(self, event):
        if time.time() - self.lastModified < .25:
            return

        folder = os.path.dirname(self.fname)
        try:
            nEncounters = self.getEncounters()
        except AttributeError:
            return
        self.lastModified = time.time()

        p = probShiny(nEncounters, self.odds, self.nShines)

        self.textFiles(p, nEncounters, folder)
        self.plot(p, nEncounters, folder)

    def textFiles(self, p, nEncounters, folder):
        print(
            f'P = {100 * p:.2f}% ({self.oddsStr} base odds) {"(" + str(self.nShines) + " total shines)" if self.nShines > 1 else ""}')
        with open(os.path.join(folder, 'odds.txt'), 'w+') as f:
            f.write(f'Current Odds: {100 * p:.2f}%')

        if p < .5:
            numToReach = self.comb[:, 0][self.comb[:, 1] > .5][0] - nEncounters
            percToReach = 50
        elif p < .75:
            numToReach = self.comb[:, 0][self.comb[:, 1] > .75][0] - nEncounters
            percToReach = 75
        elif p < .9:
            numToReach = self.comb[:, 0][self.comb[:, 1] > .9][0] - nEncounters
            percToReach = 90
        elif p < .999:
            numToReach = self.comb[:, 0][self.comb[:, 1] > .999][0] - nEncounters
            percToReach = 99.9

        with open(os.path.join(folder, 'next.txt'), 'w+') as f:
            if p < .999:
                print(f"Number of encounters until >{percToReach}%: {int(numToReach)}")
                f.write(f"Number of encounters until >{percToReach}%: {int(numToReach)}")
            else:
                f.write(f"Gotta get some better luck")

    def plot(self, p, nEncounters, folder):
        self.ax.cla()
        self.ax.plot(self.n, self.probs, label="Bimodal CDF")
        self.ax.hlines([p * 100], xmin=0, xmax=max(self.n), colors="r", label="Current Value", linestyles='dashed')
        self.ax.vlines([nEncounters], ymin=0, ymax=100, colors="r", linestyles='dashed')
        # ax.hlines([pNext], xmin=0, xmax=max(n), colors='r', linestyles='dashed', label="Next Value")
        # ax.vlines([nEncounters+intbox.value], ymin=0, ymax=1, colors='r', linestyles='dashed')
        self.ax.vlines([1 / self.odds], ymin=0, ymax=100, colors='g', linestyles='dashed', label="Odds")
        self.ax.set_xlabel("Number of Encounters")
        self.ax.set_ylabel(f"Chance to Have Gotten Shiny Mon")
        self.ax.legend()
        self.ax.grid()
        self.ax.set_xscale("log")
        self.ax.set_xlim([max(nEncounters / 10, 1), np.amax(self.n)])
        self.ax.yaxis.set_major_formatter(mtick.PercentFormatter())

        plt.tight_layout()
        plt.savefig(os.path.join(folder, 'encGraph.png'))
        os.utime(os.path.join(folder, 'encGraph.png'), (time.time(), time.time()))
        self.ax.cla()


def main():
    args = parse_arguments()
    event_handler = ShinyCounterHandler(args)
    observer = Observer()
    observer.schedule(event_handler, path=os.path.dirname(args.encounter_fname), recursive=False)
    # observer.schedule(event_handler, path="StreamCounters/", recursive=False)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


if __name__ == "__main__":
    main()



from botCommands import Bot
import argparse


def parse_arguments():
    parser = argparse.ArgumentParser(description='Twitch Bot.')
    parser.add_argument('token', metavar='oauth:xxx', type=str,
                        help='The twitch authentication token')
    parser.add_argument('client_id', metavar='xxx', type=str,
                        help='The twitch app client ID.')
    parser.add_argument('channel', metavar='Channel Name', type=str,
                        help='The channel name for the bot to be in.')

    return parser.parse_args()


def main(args=None):
    if args is None:
        args = parse_arguments()
    bot = Bot(
        # set up the bot
        token=args.token,
        client_id=args.client_id,
        prefix="!",
        initial_channels=[args.channel]
    )

    bot.run()


if __name__ == "__main__":
    args = parse_arguments()
    main(args)
